;#########################################################################
; BIOE Module for the ATMega16 Controller
; Copyright (C) 2008 Klaus Weichinger
;
; License: CC-GNU GPL
; http://creativecommons.org/licenses/GPL/2.0/
;#########################################################################
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor,
; Boston, MA  02110-1301, USA.
;#########################################################################

;{description}
;=============================================
; DTN106 - Universalansteuerung
;          4x10bit ADC
;          4xModellbauservo
;=============================================
;
;  +---------------------------------+
;  |DTN 106                      AREF+-
;  |      IN          OUT        P15 +- ADC4 In
;  |EP2               SERVO1     P14 +- ADC3 In
;  |EP3               SERVO2     P13 +- ADC2 In
;  |EP4               SERVO3     P12 +- ADC1 In
;  |EP5               SERVO4     P11 +- RESET
;  |EP6   ADC1                   P10 +- B
;  |EP7   ADC2                   P9  +- A
;  |EP8   ADC3                   GND +-
;  |EP9   ADC4                   P8  +- OUT1-B
;  |EP10              MOT        P7  +- OUT1-A
;  |EP11  POS                    RES +-
;  |EP12  SER0                   P6  +- LED (ADC-Sample Impuls Output)
;  |EP13  SER1                   VCC +-
;  |EP14                         P5  +- SERIAL-IN (in Planung)
;  |                             P4  +- SERVO4
;  |                             P3  +- SERVO3
;  |                             P2  +- SERVO2
;  |                             P1  +- SERVO1
;  |                             P0  +- PWM1
;  +---------------------------------+
;
;  Analogwertmessung mit ADCX:
;  ===========================
;    ADCX=1023 ... bei 5V
;    ADCX=0    ... bei 0V
;
;  Motoransteuerung:
;  =================
;  MOT1=PWM1+(OUT1-A)*256+(OUT1-B*512)
;    PWM1   = [0,...,255]
;    OUT1-A = [0,1]
;    OUT1-B = [0,1]
;
;  Servoansteuerung:
;  =================
;  Ein normaler Servo bentigt ein Impulssignal von
;  20ms Periodendauer und 1..2ms High-Phase. Die
;  Dauer der High-Phase gibt direkt die Position an.
;
;  Mit den Werten in den Endpunkten (SERVOX) kann man diese
;  HIGH-Zeitdauer einstellen, wobei
;    0=0ms  und
;    39936=2ms
;  bedeutet.
;
;  Inkrementaldecoder:
;  ===================
;  0 Punkt     POS=32768
;  Aktiv auf fallende und steigende Flanke von B
;  Zhlrichtung wird anhand von A entschieden.
;  RESET=HIGH  POS=32768 (synchron mit CLK)
;
;  Serielle Schnittstelle fr die Dateneingabe (in Planung, mom. nicht aktiv)
;  ===========================================
;  Empfangspaket:
;  [BYTE1][BYTE2][BYTE3]
;  BITS     |7     |6     |5     |4     |3     |2     |1     |0     |
;  ---------+------+------+------+------+------+------+------+------+
;  BYTE1:   |1     |SER0/1|BIT15 |BIT14 |BIT13 |BIT12 |BIT11 |BIT10 |
;  BYTE2:   |0     |SER0/1|BIT9  |BIT8  |BIT7  |BIT6  |BIT5  |BIT4  |
;  BYTE3:   |0     |SER0/1|BIT3  |BIT2  |BIT1  |BIT0  |SUM1  |SUM0  |
;  
;  SUM=Summe aller Bits von BYTE1..3.
;  SUM1=BIT 1 von SUM (2^1)
;  SUM2=BIT 0 von SUM (2^0)
;
;
;  Abhngigkeiten der Implementierung:
;  ===================================
;    - Verwendung der selben INT2-Interruptserviceroutine von DTN100
;
;{end}
do_DTN106:
	CPI		i,0
	BRNE	init_DTN106
	RJMP	process_DTN106
init_DTN106:
	PUSH	temp
	IN		temp,SREG
	PUSH	i
	PUSH	A
	PUSH	B
	PUSH	C

	; Ports als Eingnge
	CBI		P9_DIR,P9_PIN
	CBI		P10_DIR,P10_PIN
	CBI		P11_DIR,P11_PIN

	; Eingnge Pull-UPs aktivieren
	SBI		P9_OUT,P9_PIN
	SBI		P10_OUT,P10_PIN
	SBI		P11_OUT,P11_PIN

	; INT2 Initialisieren

	LDI		i,0x80
	MOV		DEVDATH,i
	MOV		DEVDATL,zero

	LDI		A,11
	MOV		B,DEVDATH
	MOV		C,DEVDATL
	CALL	setTXEP

	IN		i,MCUCSR		; INT2 aktiv auf steigende Flanke
	ORI		i,(1<<ISC2)
	OUT		MCUCSR,i

	IN		i,GIFR
	ORI		i,(1<<INTF2)	; INT2 Flag lschen
	OUT		GIFR,i

	IN		i,GICR
	ORI		i,(1<<INTF2)	; INT2 aktivieren
	OUT		GICR,i

	MOV		DEVFLAG2,ZERO

	IN		i,PORTD
	ANDI	i,~0b01110010	; Ausgnge=0 setzen
	OUT		PORTD,i

	IN		i,DDRD
	ORI		i,0b01110010	; Ausgnge setzen
	ANDI	i,~0b00000000	; Eingnge setzen
	OUT		DDRD,i

	LDI		i,(1<<WGM11) | (0<<WGM10) | (0<<COM1A1) | (0<<COM1A0) | (0<<COM1B1) | (0<<COM1B0)
	OUT		TCCR1A,i

	LDI		i,(1<<WGM13)|(1<<WGM12) | (0<<CS12) | (0<<CS11) | (1<<CS10)
	OUT		TCCR1B,i

	LDI		i,0x9C
	OUT		ICR1H,i
	LDI		i,0x00
	OUT		ICR1L,i
	
	LDI		i,0
	MOV		DEVFLAG,i

	IN		i,TIMSK
	ORI		i,(1<<TOIE1)|(1<<OCIE1A)
	OUT		TIMSK,i

	; ADC initialisieren
    LDI		i, 0 | (1<<REFS0)  ; interne Referenzspannung 5V, Kanal-0
    OUT		ADMUX, i
    LDI		i, (1<<ADEN) | (1<<ADPS2) | (0<<ADPS1) | (1<<ADPS0) ;  Division Faktor 32
    OUT		ADCSRA, i
	SBI		ADCSRA,ADSC		; ADC starten
	LDI		DEVFLAG2,0

; 	; Phasencorrekte PWM 8bit
; 	LDI		i,(0<<WGM01) | (1<<WGM00) | (1<<COM01) | (0<<COM00) | (0<<CS02) | (0<<CS01) | (1<<CS00)
; 	OUT		TCCR0,i

	; Phasencorrekte PWM 8bit
	LDI		i,(0<<WGM21) | (1<<WGM20) | (1<<COM21) | (0<<COM20) | (0<<CS22) | (0<<CS21) | (1<<CS20)
	OUT		TCCR2,i


	; Ports als Ausgnge
	SBI		P0_DIR,P0_PIN
	SBI		P6_DIR,P6_PIN ; Impulse Output
	IN		i,DDRB
	ORI		i,0b01100000	; Ausgnge setzen
	OUT		DDRB,i
	IN		i,PORTB
	ANDI	i,~0b01100000	; Ausgnge=0
	OUT		PORTB,i
	
	POP		C
	POP		B
	POP		A
	POP		i
	OUT		SREG,temp
	POP		temp
	JMP	initDeviceType_end

process_DTN106:

	CPI		RXEPFLAG,10
	BRNE	process_DTN106_not_EP10
	LDI		RXEPFLAG,$FF	; FLAG lschen
	LDI		A,10
	CALL	getRXEP
	IN		i,PORTB
	ANDI	i,~0b01100000 ; Ausgnge auf 0 setzen
	SBRC	B,0
	ORI		i,0b00100000
	SBRC	B,1
	ORI		i,0b01000000
	OUT		PORTB,i
	OUT		OCR2,C
process_DTN106_not_EP10:

; 	CPI		RXEPFLAG,11
; 	BRNE	process_DTN106_not_EP11
; 	LDI		RXEPFLAG,$FF	; FLAG lschen
; 	LDI		A,11
; 	CALL	getRXEP
; 	IN		i,PORTB
; 	ANDI	i,~0b00000101 ; Ausgnge auf 0 setzen
; 	SBRC	B,0
; 	ORI		i,0b00000100
; 	SBRC	B,1
; 	ORI		i,0b00000001
; 	OUT		PORTB,i
; 	OUT		OCR0,C
; process_DTN106_not_EP11:

    SBIC	ADCSRA, ADSC
    RJMP process_DTN106_adc_end
	; ADC ist jetzt fertig und kann verarbeitet werden

	MOV		i,DEVFLAG2
 	CPI		i,0
	BRNE	process_DTN106_adc_0_end
	SBI		P6_OUT,P6_PIN
	LDI		A,6
	IN		C,ADCL
	IN		B,ADCH
	CALL	setTXEP

    LDI		i, 1  ; Kanal-1
	MOV		DEVFLAG2,i
	ORI		i,(1<<REFS0); interne Referenzspannung 5V
    OUT		ADMUX, i
	SBI		ADCSRA,ADSC		; ADC starten
	RJMP	process_DTN106_adc_end
process_DTN106_adc_0_end:

 	CPI		i,1
	BRNE	process_DTN106_adc_1_end
	CBI		P6_OUT,P6_PIN
	LDI		A,7
	IN		C,ADCL
	IN		B,ADCH
	CALL	setTXEP

    LDI		i, 2  ; Kanal-2
	MOV		DEVFLAG2,i
	ORI		i,(1<<REFS0); interne Referenzspannung 5V
    OUT		ADMUX, i
	SBI		ADCSRA,ADSC		; ADC starten
	RJMP	process_DTN050_adc_end
process_DTN106_adc_1_end:

 	CPI		i,2
	BRNE	process_DTN106_adc_2_end
	LDI		A,8
	IN		C,ADCL
	IN		B,ADCH
	CALL	setTXEP

    LDI		i, 3  ; Kanal-3
	MOV		DEVFLAG2,i
	ORI		i,(1<<REFS0); interne Referenzspannung 5V
    OUT		ADMUX, i
	SBI		ADCSRA,ADSC		; ADC starten
	CBI		P6_OUT,P6_PIN
	RJMP	process_DTN050_adc_end
process_DTN106_adc_2_end:

 	CPI		i,3
	BRNE	process_DTN106_adc_3_end
	LDI		A,9
	IN		C,ADCL
	IN		B,ADCH
	CALL	setTXEP

	LDI		A,11
	MOV		B,DEVDATH
	MOV		C,DEVDATL
	CALL	setTXEP

    LDI		i, 0  ; Kanal-0
	MOV		DEVFLAG2,i
	ORI		i,(1<<REFS0); interne Referenzspannung 5V
    OUT		ADMUX, i
	SBI		ADCSRA,ADSC		; ADC starten
	CBI		P6_OUT,P6_PIN
	RJMP	process_DTN050_adc_end
process_DTN106_adc_3_end:

process_DTN106_adc_end:
	JMP	processDeviceType_end
