;#########################################################################
; BIOE Module for the ATMega16 Controller
; Copyright (C) 2008 Klaus Weichinger
;
; License: CC-GNU GPL
; http://creativecommons.org/licenses/GPL/2.0/
;#########################################################################
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor,
; Boston, MA  02110-1301, USA.
;#########################################################################

;{description}
;=============================================
; Streckensimulator MASTER
; mit synchroner serieller Schnittstelle
;=============================================
;
;  +---------------------------------+
;  |DTN 200                      AREF+-
;  |      IN          OUT        P15 +- 
;  |EP2                          P14 +- 
;  |EP3                          P13 +- 
;  |EP4                          P12 +- 
;  |EP5                          P11 +- CLK (Output)
;  |EP6                          P10 +- 
;  |EP7                          P9  +- 
;  |EP8                          GND +-
;  |EP9                          P8  +- 
;  |EP10                         P7  +- 
;  |EP11                         RES +-
;  |EP12                         P6  +- 
;  |EP13                         VCC +-
;  |EP14                         P5  +- RXD
;  |                             P4  +- TXD
;  |                             P3  +- 
;  |                             P2  +- 
;  |                             P1  +- 
;  |                             P0  +- 
;  +---------------------------------+
;
;  Die Settingsregister S0...S7 definieren, ob die Endpunkte
;  EP2...EP9 deaktiviert, gelesen und/ oder geschrieben werden.
;  Die Endpunkte 10-14 werden in dieser Konfiguration nicht verwendet.
;  
;  Sx=0  ... EP(x+2) ist deaktiviert
;  Sx=1  ... EP(x+2) wird gelesen (Input)
;  Sx=2  ... EP(x+2) wird geschrieben (Output)
;  Sx=3  ... EP(x+2) wird gelesen und geschrieben
;
;  Durch die Verwendung der Settingsregister kann der
;  bertragungskanal optimal ausgenutzt werden, so dass eine
;  maximale Refreshgeschwindigkeit der Endpunkte erzielt wird.
; 
;  Default Setting ist:
;    S0=2
;    S1=2
;    S2=1
;    S3=1
;    S4=0
;    S5=0
;    S6=0
;    S7=0
;
; Zur Abschtzung der Dauer eines Refreshzykluses kann folgende
; Formel verwendet werden:
; N...Anzahl der zu lesenden und schreibenden Endpunkte
; T=33us+N*60us
;{end}

do_DTN200:
	CPI		i,0
	BRNE	init_DTN200
	RJMP	process_DTN200
init_DTN200:

	PUSH	temp
	IN		temp,SREG
	PUSH	i

	LDI		i,0
	OUT 	UBRRH, i
	LDI		i,7
    OUT 	UBRRL, i

	SBI		P6_DIR,P6_PIN ; Impulse Output

    ; Frame-Format: 8 Bit
	SBI		P11_DIR, P11_PIN		; Master mode
    LDI 	i, (1<<UMSEL)|(0<<UCPOL)|(3<<UCSZ0)|(1<<URSEL)|(0<<USBS)
    OUT 	UCSRC, i

    SBI 	UCSRB, TXEN 			; USRT transmit enable
	SBI 	UCSRB, RXEN             ; USRT receive enable

	LDI		DEVFLAG,0

	LDI		i,2
	STS		S0,i
	STS		S1,i
	LDI		i,1
	STS		S2,i
	STS		S3,i
	LDI		i,0
	STS		S4,i
	STS		S5,i
	STS		S6,i
	STS		S7,i

	POP		i
	OUT		SREG,temp
	POP		temp

	JMP	initDeviceType_end


process_DTN200:
	MOV		A,DEVFLAG
	INC		A
	INC		A

	RCALL	DTN200_processEndpoint
	CBI		P6_OUT,P6_PIN	;  EP Refresh Impulse setzen (nur zur Bestimmung der Refreshrate)

	INC		DEVFLAG
	CPI		DEVFLAG,8
	BRNE	process_DTN200_end
	LDI		DEVFLAG,0
	SBI		P6_OUT,P6_PIN ;  EP Refresh Impulse lschen (nur zur Bestimmung der Refreshrate)
process_DTN200_end:
	JMP	processDeviceType_end


DTN200_processEndpoint:
	PUSH	A
	PUSH	B
	PUSH	C
	PUSH	i

	MOV		j,ZERO

	CPI		A,2
	BRNE	DTN200_NOTEP2
	LDS		j,S0
DTN200_NOTEP2:

	CPI		A,3
	BRNE	DTN200_NOTEP3
	LDS		j,S1
DTN200_NOTEP3:

	CPI		A,4
	BRNE	DTN200_NOTEP4
	LDS		j,S2
DTN200_NOTEP4:

	CPI		A,5
	BRNE	DTN200_NOTEP5
	LDS		j,S3
DTN200_NOTEP5:

	CPI		A,6
	BRNE	DTN200_NOTEP6
	LDS		j,S4
DTN200_NOTEP6:

	CPI		A,7
	BRNE	DTN200_NOTEP7
	LDS		j,S5
DTN200_NOTEP7:

	CPI		A,8
	BRNE	DTN200_NOTEP8
	LDS		j,S6
DTN200_NOTEP8:

	CPI		A,9
	BRNE	DTN200_NOTEP9
	LDS		j,S7
DTN200_NOTEP9:

	MOV		i,j
	ANDI	i,0b00000010
	CPI		i,0b00000010
	BRNE	DTN200_dontwrite

	CALL	getRXEP

	LDI		i, 'W'
	RCALL	DTN200_sendPAUSE;
	RCALL	DTN200_sendByte
	MOV		i, A
	RCALL	DTN200_sendPAUSE;
	RCALL	DTN200_sendASCII

	MOV		i, B
	SWAP	i
	ANDI	i, 0x0F
	RCALL	DTN200_sendPAUSE;
	RCALL	DTN200_sendASCII

	MOV		i, B
	ANDI	i, 0x0F
	RCALL	DTN200_sendPAUSE;
	RCALL	DTN200_sendASCII

	MOV		i, C
	SWAP	i
	ANDI	i, 0x0F
	RCALL	DTN200_sendPAUSE;
	RCALL	DTN200_sendASCII

	MOV		i, C
	ANDI	i, 0x0F
	RCALL	DTN200_sendPAUSE;
	RCALL	DTN200_sendASCII
DTN200_dontwrite:

	MOV		i,j
	ANDI	i,0b00000001
	CPI		i,0b00000001
	BRNE	DTN200_dontread

	LDI		i, 'R'
	RCALL	DTN200_sendPAUSE;
	RCALL	DTN200_sendByte

	MOV		i, A
	RCALL	DTN200_sendPAUSE;
	RCALL	DTN200_sendASCII

	RCALL	DTN200_readASCII
	ANDI	i,0x0F
	SWAP	i
	MOV		B,i

	RCALL	DTN200_readASCII
	ANDI	i,0x0F
	OR		B,i

	RCALL	DTN200_readASCII
	ANDI	i,0x0F
	SWAP	i
	MOV		C,i

	RCALL	DTN200_readASCII
	ANDI	i,0x0F
	OR		C,i
	
 	CALL	setTXEP
DTN200_dontread:

	POP		i
	POP		C
	POP		B
	POP		A
	RET

DTN200_readASCII:
	SBIS 	UCSRA, RXC              ; return if nothing was received
	JMP	DTN200_readASCII
	IN 		i, UDR 					; read received Byte from UART Data register              
	CPI		i, 65
	BRLO	DTN200_readASCII_NUMBER
	SUBI	i,55
	RET
DTN200_readASCII_NUMBER:
	SUBI	i, 48
	RET

DTN200_sendASCII:
	CPI		i,10
	BRSH	DTN200_ABCDEF
	SUBI	i,-48
	RJMP	DTN200_weiter_r
DTN200_ABCDEF:
	SUBI	i,-55
DTN200_weiter_r:
DTN200_UARTsend:
    SBIS UCSRA,UDRE                  ; wait until UDR is ready for next byte
    RJMP DTN200_UARTsend
    OUT UDR, i
    RET

DTN200_sendByte:
    SBIS 	UCSRA,UDRE                  ; wait until UDR is ready for next byte
    RJMP 	DTN200_sendByte
    OUT 	UDR, i
    RET

DTN200_sendPAUSE:
	PUSH	i
	LDI		i,20
DTN200_sendPAUSE_wait:
	DEC		i
	CPI		i,0
	BRNE	DTN200_sendPAUSE_wait
	POP		i
	RET



